@@ -91,22 +91,23 @@ fwupd_enums_func(void)
 	}
 
 	/* bitfield */
+	for (guint64 i = 1; i <= FWUPD_DEVICE_FLAG_INSTALL_SKIP_VERSION_CHECK; i *= 2) {
+		const gchar *tmp = fwupd_device_flag_to_string(i);
+		if (tmp == NULL)
+			continue;
+		g_assert_cmpint(fwupd_device_flag_from_string(tmp), ==, i);
+	}
 	for (guint64 i = 1; i <= FWUPD_DEVICE_PROBLEM_IN_USE; i *= 2) {
 		const gchar *tmp = fwupd_device_problem_to_string(i);
 		g_assert_cmpstr(tmp, !=, NULL);
 		g_assert_cmpint(fwupd_device_problem_from_string(tmp), ==, i);
 	}
-	for (guint64 i = 1; i <= FWUPD_PLUGIN_FLAG_MEASURE_SYSTEM_INTEGRITY; i *= 2) {
+	for (guint64 i = 1; i <= FWUPD_PLUGIN_FLAG_TEST_ONLY; i *= 2) {
 		const gchar *tmp = fwupd_plugin_flag_to_string(i);
 		g_assert_cmpstr(tmp, !=, NULL);
 		g_assert_cmpint(fwupd_plugin_flag_from_string(tmp), ==, i);
 	}
-	for (guint64 i = 1; i <= FWUPD_FEATURE_FLAG_SHOW_PROBLEMS; i *= 2) {
-		const gchar *tmp = fwupd_feature_flag_to_string(i);
-		g_assert_cmpstr(tmp, !=, NULL);
-		g_assert_cmpint(fwupd_feature_flag_from_string(tmp), ==, i);
-	}
-	for (guint64 i = 1; i <= FWUPD_FEATURE_FLAG_ALLOW_AUTHENTICATION; i *= 2) {
+	for (guint64 i = 1; i <= FWUPD_FEATURE_FLAG_REQUESTS_NON_GENERIC; i *= 2) {
 		const gchar *tmp = fwupd_feature_flag_to_string(i);
 		g_assert_cmpstr(tmp, !=, NULL);
 		g_assert_cmpint(fwupd_feature_flag_from_string(tmp), ==, i);
@@ -116,7 +117,7 @@ fwupd_enums_func(void)
 		g_assert_cmpstr(tmp, !=, NULL);
 		g_assert_cmpint(fwupd_release_flag_from_string(tmp), ==, i);
 	}
-	for (guint64 i = 1; i <= FWUPD_REQUEST_FLAG_ALLOW_GENERIC_IMAGE; i *= 2) {
+	for (guint64 i = 1; i <= FWUPD_REQUEST_FLAG_NON_GENERIC_IMAGE; i *= 2) {
 		const gchar *tmp = fwupd_request_flag_to_string(i);
 		g_assert_cmpstr(tmp, !=, NULL);
 		g_assert_cmpint(fwupd_request_flag_from_string(tmp), ==, i);
@@ -127,6 +128,12 @@ fwupd_enums_func(void)
 			break;
 		g_assert_cmpint(fwupd_remote_flag_from_string(tmp), ==, i);
 	}
+	for (guint64 i = 1; i <= FWUPD_INSTALL_FLAG_IGNORE_REQUIREMENTS; i *= 2) {
+		const gchar *tmp = fwupd_install_flags_to_string(i);
+		if (tmp == NULL)
+			continue;
+		g_assert_cmpint(fwupd_install_flags_from_string(tmp), ==, i);
+	}
 }
 
 static void
@@ -162,6 +169,7 @@ fwupd_release_func(void)
 	fwupd_release_set_homepage(release1, "homepage");
 	fwupd_release_set_details_url(release1, "details_url");
 	fwupd_release_set_source_url(release1, "source_url");
+	fwupd_release_set_sbom_url(release1, "sbom_url");
 	fwupd_release_set_version(release1, "version");
 	fwupd_release_set_vendor(release1, "vendor");
 	fwupd_release_set_size(release1, 1234);
@@ -206,6 +214,7 @@ fwupd_release_func(void)
 	g_assert_cmpstr(fwupd_release_get_homepage(release2), ==, "homepage");
 	g_assert_cmpstr(fwupd_release_get_details_url(release2), ==, "details_url");
 	g_assert_cmpstr(fwupd_release_get_source_url(release2), ==, "source_url");
+	g_assert_cmpstr(fwupd_release_get_sbom_url(release2), ==, "sbom_url");
 	g_assert_cmpstr(fwupd_release_get_version(release2), ==, "version");
 	g_assert_cmpstr(fwupd_release_get_vendor(release2), ==, "vendor");
 	g_assert_cmpint(fwupd_release_get_size(release2), ==, 1234);
@@ -260,6 +269,7 @@ fwupd_release_func(void)
 				    "  Homepage:             homepage\n"
 				    "  DetailsUrl:           details_url\n"
 				    "  SourceUrl:            source_url\n"
+				    "  SbomUrl:              sbom_url\n"
 				    "  Urgency:              medium\n"
 				    "  Vendor:               vendor\n"
 				    "  Flags:                is-upgrade\n"
@@ -381,6 +391,36 @@ fwupd_plugin_func(void)
 	g_assert_true(ret);
 }
 
+static void
+fwupd_remote_func(void)
+{
+	g_autofree gchar *uri1 = NULL;
+	g_autofree gchar *uri2 = NULL;
+	g_autofree gchar *uri3 = NULL;
+	g_autoptr(FwupdRemote) remote = fwupd_remote_new();
+	g_autoptr(GError) error = NULL;
+
+	uri1 = fwupd_remote_build_firmware_uri(remote,
+					       "https://example.org/downloads/foo.cab",
+					       &error);
+	g_assert_no_error(error);
+	g_assert_cmpstr(uri1, ==, "https://example.org/downloads/foo.cab");
+
+	fwupd_remote_set_firmware_base_uri(remote, "https://example.org/mirror");
+	uri2 = fwupd_remote_build_firmware_uri(remote,
+					       "https://example.org/downloads/foo.cab",
+					       &error);
+	g_assert_no_error(error);
+	g_assert_cmpstr(uri2, ==, "https://example.org/mirror/foo.cab");
+
+	fwupd_remote_set_username(remote, "admin");
+	uri3 = fwupd_remote_build_firmware_uri(remote,
+					       "https://example.org/downloads/foo.cab",
+					       &error);
+	g_assert_no_error(error);
+	g_assert_cmpstr(uri3, ==, "https://admin@example.org/mirror/foo.cab/auth");
+}
+
 static void
 fwupd_request_func(void)
 {
@@ -470,6 +510,206 @@ fwupd_device_filter_func(void)
 					       FWUPD_DEVICE_FLAG_ANOTHER_WRITE_REQUIRED));
 }
 
+static void
+fwupd_client_api_undefined_setter(void)
+{
+#if GLIB_CHECK_VERSION(2, 74, 0)
+	if (g_test_subprocess()) {
+		g_autoptr(FwupdClient) client = fwupd_client_new();
+		GValue value_bool = G_VALUE_INIT;
+
+		g_value_init(&value_bool, G_TYPE_BOOLEAN);
+		g_object_set_property(G_OBJECT(client), "battery-adapter", &value_bool);
+	} else {
+		g_test_trap_subprocess("/fwupd/client_api{undefined_setter}",
+				       0,
+				       G_TEST_SUBPROCESS_DEFAULT);
+		g_test_trap_assert_failed();
+		g_test_trap_assert_stderr(
+		    "*GLib-GObject-CRITICAL*has no property named 'battery-adapter'*");
+	}
+#else
+	g_test_skip("Trap handling requires glib 2.74");
+#endif
+}
+
+static void
+fwupd_client_api_undefined_getter(void)
+{
+#if GLIB_CHECK_VERSION(2, 74, 0)
+	if (g_test_subprocess()) {
+		g_autoptr(FwupdClient) client = fwupd_client_new();
+		GValue value_bool = G_VALUE_INIT;
+
+		g_value_init(&value_bool, G_TYPE_BOOLEAN);
+		g_object_get_property(G_OBJECT(client), "battery-adapter", &value_bool);
+	} else {
+		g_test_trap_subprocess("/fwupd/client_api{undefined_getter}",
+				       0,
+				       G_TEST_SUBPROCESS_DEFAULT);
+		g_test_trap_assert_failed();
+		g_test_trap_assert_stderr(
+		    "*GLib-GObject-CRITICAL*has no property named 'battery-adapter'*");
+	}
+#else
+	g_test_skip("Trap handling requires glib 2.74");
+#endif
+}
+
+static void
+fwupd_client_api_ro_props(void)
+{
+#if GLIB_CHECK_VERSION(2, 74, 0)
+	const gchar *props[] = {"daemon-version", "tainted", "interactive", "only-trusted", NULL};
+
+	for (guint i = 0; i < G_N_ELEMENTS(props); i++) {
+		if (g_test_subprocess()) {
+			g_autoptr(FwupdClient) client = fwupd_client_new();
+			GValue value_bool = G_VALUE_INIT;
+
+			g_value_init(&value_bool, G_TYPE_BOOLEAN);
+			g_object_set_property(G_OBJECT(client), props[i], &value_bool);
+		} else {
+			g_test_trap_subprocess("/fwupd/client_api{ro_props}",
+					       0,
+					       G_TEST_SUBPROCESS_DEFAULT);
+			g_test_trap_assert_failed();
+			g_test_trap_assert_stderr(
+			    "*GLib-GObject-CRITICAL*property*is not writable*");
+		}
+	}
+#else
+	g_test_skip("Trap handling requires glib 2.74");
+#endif
+}
+
+static void
+fwupd_client_api(void)
+{
+	gboolean ret;
+	const gchar *tmp = "1234567890abcdef";
+	GValue value_str = G_VALUE_INIT;
+	GValue value_int = G_VALUE_INIT;
+	GValue value_bool = G_VALUE_INIT;
+	g_autoptr(FwupdClient) client = fwupd_client_new();
+	g_autoptr(GError) error = NULL;
+
+	g_value_init(&value_str, G_TYPE_STRING);
+	g_value_init(&value_int, G_TYPE_INT);
+	g_value_init(&value_bool, G_TYPE_BOOLEAN);
+	g_value_set_string(&value_str, tmp);
+
+	ret = fwupd_client_get_only_trusted(client);
+	g_assert_false(ret);
+	ret = fwupd_client_get_daemon_interactive(client);
+	g_assert_false(ret);
+	ret = fwupd_client_get_tainted(client);
+	g_assert_false(ret);
+
+	/* set the version multiple times */
+	fwupd_client_set_daemon_version(client, "1.2.3");
+	g_assert_cmpstr(fwupd_client_get_daemon_version(client), ==, "1.2.3");
+	fwupd_client_set_daemon_version(client, "1.2.4");
+	g_assert_cmpstr(fwupd_client_get_daemon_version(client), ==, "1.2.4");
+	fwupd_client_set_daemon_version(client, "1.2.4");
+
+	/* set host security ID multiple times */
+	g_object_set_property(G_OBJECT(client), "host-security-id", &value_str);
+	g_assert_cmpstr(fwupd_client_get_host_security_id(client), ==, tmp);
+	g_object_set_property(G_OBJECT(client), "host-security-id", &value_str);
+
+	/* set host machine ID multiple times */
+	g_object_set_property(G_OBJECT(client), "host-machine-id", &value_str);
+	g_assert_cmpstr(fwupd_client_get_host_machine_id(client), ==, tmp);
+	g_object_set_property(G_OBJECT(client), "host-machine-id", &value_str);
+
+	/* set host product ID and product vendor multiple times */
+	tmp = "Acme";
+	g_value_set_string(&value_str, tmp);
+	g_object_set_property(G_OBJECT(client), "host-vendor", &value_str);
+	g_assert_cmpstr(fwupd_client_get_host_vendor(client), ==, tmp);
+	g_object_set_property(G_OBJECT(client), "host-vendor", &value_str);
+
+	tmp = "Anvil";
+	g_value_set_string(&value_str, tmp);
+	g_object_set_property(G_OBJECT(client), "host-product", &value_str);
+	g_assert_cmpstr(fwupd_client_get_host_product(client), ==, tmp);
+	g_object_set_property(G_OBJECT(client), "host-product", &value_str);
+
+	/* set BKC */
+	tmp = "BKC-123";
+	g_value_set_string(&value_str, tmp);
+	g_object_set_property(G_OBJECT(client), "host-bkc", &value_str);
+	g_assert_cmpstr(fwupd_client_get_host_bkc(client), ==, tmp);
+	g_object_set_property(G_OBJECT(client), "host-bkc", &value_str);
+
+	/* verify experience with no user agent explicitly */
+	ret = fwupd_client_ensure_networking(client, &error);
+	g_assert_true(ret);
+	g_assert_no_error(error);
+
+	/* verify experience with a good user agent*/
+	fwupd_client_set_user_agent_for_package(client, "fwupd", "2.0.0");
+	ret = fwupd_client_ensure_networking(client, &error);
+	g_assert_true(ret);
+	g_assert_no_error(error);
+
+	/* set same battery level multiple times */
+	g_value_set_int(&value_int, 50);
+	g_object_set_property(G_OBJECT(client), "battery-level", &value_int);
+	g_assert_cmpint(fwupd_client_get_battery_level(client), ==, 50);
+	g_object_set_property(G_OBJECT(client), "battery-level", &value_int);
+
+	/* set same battery threshold multiple times */
+	g_value_set_int(&value_int, 20);
+	g_object_set_property(G_OBJECT(client), "battery-threshold", &value_int);
+	g_assert_cmpint(fwupd_client_get_battery_threshold(client), ==, 20);
+	g_object_set_property(G_OBJECT(client), "battery-threshold", &value_int);
+
+	/* set same status multiple times */
+	g_value_set_int(&value_int, FWUPD_STATUS_IDLE);
+	g_object_set_property(G_OBJECT(client), "status", &value_int);
+	g_assert_cmpint(fwupd_client_get_status(client), ==, FWUPD_STATUS_IDLE);
+	g_object_set_property(G_OBJECT(client), "status", &value_int);
+
+	/* set same percentage multiple times */
+	g_value_set_int(&value_int, 50);
+	g_object_set_property(G_OBJECT(client), "percentage", &value_int);
+	g_assert_cmpint(fwupd_client_get_percentage(client), ==, 50);
+	g_object_set_property(G_OBJECT(client), "percentage", &value_int);
+
+	/* set all properties */
+	g_value_set_int(&value_int, 0);
+	g_object_set_property(G_OBJECT(client), "status", &value_int);
+	g_object_set_property(G_OBJECT(client), "percentage", &value_int);
+	g_object_set_property(G_OBJECT(client), "host-bkc", &value_str);
+	g_object_set_property(G_OBJECT(client), "host-vendor", &value_str);
+	g_object_set_property(G_OBJECT(client), "host-product", &value_str);
+	g_object_set_property(G_OBJECT(client), "host-machine-id", &value_str);
+	g_object_set_property(G_OBJECT(client), "host-security-id", &value_str);
+	g_object_set_property(G_OBJECT(client), "battery-level", &value_int);
+	g_object_set_property(G_OBJECT(client), "battery-threshold", &value_int);
+
+	/* read all properties */
+	g_object_get_property(G_OBJECT(client), "status", &value_int);
+	g_object_get_property(G_OBJECT(client), "tainted", &value_bool);
+	g_object_get_property(G_OBJECT(client), "interactive", &value_bool);
+	g_object_get_property(G_OBJECT(client), "percentage", &value_int);
+	g_object_get_property(G_OBJECT(client), "daemon-version", &value_str);
+	g_object_get_property(G_OBJECT(client), "host-bkc", &value_str);
+	g_object_get_property(G_OBJECT(client), "host-vendor", &value_str);
+	g_object_get_property(G_OBJECT(client), "host-product", &value_str);
+	g_object_get_property(G_OBJECT(client), "host-machine-id", &value_str);
+	g_object_get_property(G_OBJECT(client), "host-security-id", &value_str);
+	g_object_get_property(G_OBJECT(client), "only-trusted", &value_bool);
+	g_object_get_property(G_OBJECT(client), "battery-level", &value_int);
+	g_object_get_property(G_OBJECT(client), "battery-threshold", &value_int);
+
+	g_value_unset(&value_str);
+	g_value_unset(&value_int);
+	g_value_unset(&value_bool);
+}
+
 static void
 fwupd_common_history_report_func(void)
 {
@@ -486,7 +726,7 @@ fwupd_common_history_report_func(void)
 	fwupd_device_add_checksum(dev, "beefdead");
 	fwupd_device_add_guid(dev, "2082b5e0-7a64-478a-b1b2-e3404fab6dad");
 	fwupd_device_add_protocol(dev, "org.hughski.colorhug");
-	fwupd_device_set_plugin(dev, "colorhug");
+	fwupd_device_set_plugin(dev, "hughski_colorhug");
 	fwupd_device_set_update_error(dev, "device dead");
 	fwupd_device_set_version(dev, "1.2.3");
 	fwupd_release_add_checksum(rel, "beefdead");
@@ -527,7 +767,7 @@ fwupd_common_history_report_func(void)
 			"      \"Guid\" : [\n"
 			"        \"2082b5e0-7a64-478a-b1b2-e3404fab6dad\"\n"
 			"      ],\n"
-			"      \"Plugin\" : \"colorhug\",\n"
+			"      \"Plugin\" : \"hughski_colorhug\",\n"
 			"      \"VersionOld\" : \"1.2.3\",\n"
 			"      \"VersionNew\" : \"1.2.4\",\n"
 			"      \"Flags\" : 0,\n"
@@ -942,6 +1182,7 @@ fwupd_security_attr_func(void)
 
 	g_assert_cmpstr(fwupd_security_attr_get_appstream_id(attr1), ==, "org.fwupd.hsi.bar");
 	fwupd_security_attr_set_appstream_id(attr1, "org.fwupd.hsi.baz");
+	fwupd_security_attr_set_fwupd_version(attr1, "2.0.7");
 	g_assert_cmpstr(fwupd_security_attr_get_appstream_id(attr1), ==, "org.fwupd.hsi.baz");
 
 	fwupd_security_attr_set_level(attr1, FWUPD_SECURITY_ATTR_LEVEL_IMPORTANT);
@@ -966,6 +1207,7 @@ fwupd_security_attr_func(void)
 
 	fwupd_security_attr_set_plugin(attr1, "uefi-capsule");
 	g_assert_cmpstr(fwupd_security_attr_get_plugin(attr1), ==, "uefi-capsule");
+	g_assert_cmpstr(fwupd_security_attr_get_fwupd_version(attr1), ==, "2.0.7");
 
 	fwupd_security_attr_set_url(attr1, "https://foo.bar");
 	g_assert_cmpstr(fwupd_security_attr_get_url(attr1), ==, "https://foo.bar");
@@ -989,6 +1231,7 @@ fwupd_security_attr_func(void)
 				    "  Flags:                success\n"
 				    "  Name:                 DCI\n"
 				    "  Plugin:               uefi-capsule\n"
+				    "  Version:              2.0.7\n"
 				    "  Uri:                  https://foo.bar\n"
 				    "  Guid:                 af3fc12c-d090-5783-8a67-845b90d3cfec\n"
 				    "  KEY:                  VALUE\n",
@@ -1011,6 +1254,7 @@ fwupd_security_attr_func(void)
 				    "  Flags:                success\n"
 				    "  Name:                 DCI\n"
 				    "  Plugin:               uefi-capsule\n"
+				    "  Version:              2.0.7\n"
 				    "  Uri:                  https://foo.bar\n"
 				    "  Guid:                 af3fc12c-d090-5783-8a67-845b90d3cfec\n"
 				    "  KEY:                  VALUE\n",
@@ -1029,6 +1273,7 @@ fwupd_security_attr_func(void)
 				    "  \"HsiResult\" : \"enabled\",\n"
 				    "  \"Name\" : \"DCI\",\n"
 				    "  \"Plugin\" : \"uefi-capsule\",\n"
+				    "  \"Version\" : \"2.0.7\",\n"
 				    "  \"Uri\" : \"https://foo.bar\",\n"
 				    "  \"Flags\" : [\n"
 				    "    \"success\"\n"
@@ -1234,11 +1479,21 @@ main(int argc, char **argv)
 	g_test_add_func("/fwupd/release", fwupd_release_func);
 	g_test_add_func("/fwupd/report", fwupd_report_func);
 	g_test_add_func("/fwupd/plugin", fwupd_plugin_func);
+	g_test_add_func("/fwupd/remote", fwupd_remote_func);
 	g_test_add_func("/fwupd/request", fwupd_request_func);
 	g_test_add_func("/fwupd/device", fwupd_device_func);
 	g_test_add_func("/fwupd/device{filter}", fwupd_device_filter_func);
 	g_test_add_func("/fwupd/security-attr", fwupd_security_attr_func);
 	g_test_add_func("/fwupd/bios-attrs", fwupd_bios_settings_func);
+	g_test_add_func("/fwupd/client_api", fwupd_client_api);
+	if (g_test_undefined()) {
+		g_test_add_func("/fwupd/client_api{undefined_setter}",
+				fwupd_client_api_undefined_setter);
+		g_test_add_func("/fwupd/client_api{undefined_getter}",
+				fwupd_client_api_undefined_getter);
+		g_test_add_func("/fwupd/client_api{ro_props}", fwupd_client_api_ro_props);
+	}
+
 	if (fwupd_has_system_bus()) {
 		g_test_add_func("/fwupd/client{remotes}", fwupd_client_remotes_func);
 		g_test_add_func("/fwupd/client{devices}", fwupd_client_devices_func);
