@@ -938,7 +938,7 @@ fu_firmware_get_checksum(FuFirmware *self, GChecksumType csum_kind, GError **err
  * fu_firmware_tokenize:
  * @self: a #FuFirmware
  * @stream: a #GInputStream
- * @flags: install flags, e.g. %FWUPD_INSTALL_FLAG_FORCE
+ * @flags: #FuFirmwareParseFlags, e.g. %FWUPD_INSTALL_FLAG_FORCE
  * @error: (nullable): optional return location for an error
  *
  * Tokenizes a firmware, typically breaking the firmware into records.
@@ -953,7 +953,7 @@ fu_firmware_get_checksum(FuFirmware *self, GChecksumType csum_kind, GError **err
 gboolean
 fu_firmware_tokenize(FuFirmware *self,
 		     GInputStream *stream,
-		     FwupdInstallFlags flags,
+		     FuFirmwareParseFlags flags,
 		     GError **error)
 {
 	FuFirmwareClass *klass = FU_FIRMWARE_GET_CLASS(self);
@@ -972,7 +972,7 @@ fu_firmware_tokenize(FuFirmware *self,
  * fu_firmware_check_compatible:
  * @self: a #FuFirmware
  * @other: a #FuFirmware
- * @flags: install flags, e.g. %FWUPD_INSTALL_FLAG_FORCE
+ * @flags: #FuFirmwareParseFlags, e.g. %FWUPD_INSTALL_FLAG_FORCE
  * @error: (nullable): optional return location for an error
  *
  * Check a new firmware is compatible with the existing firmware.
@@ -984,7 +984,7 @@ fu_firmware_tokenize(FuFirmware *self,
 gboolean
 fu_firmware_check_compatible(FuFirmware *self,
 			     FuFirmware *other,
-			     FwupdInstallFlags flags,
+			     FuFirmwareParseFlags flags,
 			     GError **error)
 {
 	FuFirmwareClass *klass = FU_FIRMWARE_GET_CLASS(self);
@@ -1003,7 +1003,7 @@ static gboolean
 fu_firmware_validate_for_offset(FuFirmware *self,
 				GInputStream *stream,
 				gsize *offset,
-				FwupdInstallFlags flags,
+				FuFirmwareParseFlags flags,
 				GError **error)
 {
 	FuFirmwareClass *klass = FU_FIRMWARE_GET_CLASS(self);
@@ -1015,7 +1015,7 @@ fu_firmware_validate_for_offset(FuFirmware *self,
 
 	/* fuzzing */
 	if (!fu_firmware_has_flag(self, FU_FIRMWARE_FLAG_ALWAYS_SEARCH) &&
-	    (flags & FWUPD_INSTALL_FLAG_NO_SEARCH) > 0) {
+	    (flags & FU_FIRMWARE_PARSE_FLAG_NO_SEARCH) > 0) {
 		if (!klass->validate(self, stream, *offset, error))
 			return FALSE;
 		return TRUE;
@@ -1055,7 +1055,7 @@ fu_firmware_validate_for_offset(FuFirmware *self,
  * @self: a #FuFirmware
  * @stream: input stream
  * @offset: start offset
- * @flags: install flags, e.g. %FWUPD_INSTALL_FLAG_FORCE
+ * @flags: #FuFirmwareParseFlags, e.g. %FWUPD_INSTALL_FLAG_FORCE
  * @error: (nullable): optional return location for an error
  *
  * Parses a firmware from a stream, typically breaking the firmware into images.
@@ -1068,12 +1068,13 @@ gboolean
 fu_firmware_parse_stream(FuFirmware *self,
 			 GInputStream *stream,
 			 gsize offset,
-			 FwupdInstallFlags flags,
+			 FuFirmwareParseFlags flags,
 			 GError **error)
 {
 	FuFirmwareClass *klass = FU_FIRMWARE_GET_CLASS(self);
 	FuFirmwarePrivate *priv = GET_PRIVATE(self);
 	gsize streamsz = 0;
+	g_autoptr(GInputStream) partial_stream = NULL;
 
 	g_return_val_if_fail(FU_IS_FIRMWARE(self), FALSE);
 	g_return_val_if_fail(G_IS_INPUT_STREAM(stream), FALSE);
@@ -1136,24 +1137,36 @@ fu_firmware_parse_stream(FuFirmware *self,
 
 	/* save stream */
 	if (offset == 0) {
-		g_set_object(&priv->stream, stream);
+		partial_stream = g_object_ref(stream);
 	} else {
-		g_autoptr(GInputStream) partial_stream =
-		    fu_partial_input_stream_new(stream, offset, priv->streamsz, error);
-		if (partial_stream == NULL)
+		partial_stream = fu_partial_input_stream_new(stream, offset, priv->streamsz, error);
+		if (partial_stream == NULL) {
+			g_prefix_error(error, "failed to cut firmware: ");
 			return FALSE;
+		}
+	}
+
+	/* cache */
+	if (flags & FU_FIRMWARE_PARSE_FLAG_CACHE_BLOB) {
+		g_autoptr(GBytes) blob = NULL;
+		blob = fu_input_stream_read_bytes(partial_stream, 0x0, priv->streamsz, NULL, error);
+		if (blob == NULL)
+			return FALSE;
+		fu_firmware_set_bytes(self, blob);
+	}
+	if (flags & FU_FIRMWARE_PARSE_FLAG_CACHE_STREAM) {
 		g_set_object(&priv->stream, partial_stream);
 	}
 
 	/* optional */
 	if (klass->tokenize != NULL) {
-		if (!klass->tokenize(self, priv->stream, flags, error))
+		if (!klass->tokenize(self, partial_stream, flags, error))
 			return FALSE;
 	}
 
 	/* optional */
 	if (klass->parse != NULL)
-		return klass->parse(self, priv->stream, flags, error);
+		return klass->parse(self, partial_stream, flags, error);
 
 	/* verify alignment */
 	if (streamsz % (1ull << priv->alignment) != 0) {
@@ -1177,7 +1190,7 @@ fu_firmware_parse_stream(FuFirmware *self,
  * @self: a #FuFirmware
  * @fw: firmware blob
  * @offset: start offset, useful for ignoring a bootloader
- * @flags: install flags, e.g. %FWUPD_INSTALL_FLAG_FORCE
+ * @flags: #FuFirmwareParseFlags, e.g. %FWUPD_INSTALL_FLAG_FORCE
  * @error: (nullable): optional return location for an error
  *
  * Parses a firmware, typically breaking the firmware into images.
@@ -1190,7 +1203,7 @@ gboolean
 fu_firmware_parse_bytes(FuFirmware *self,
 			GBytes *fw,
 			gsize offset,
-			FwupdInstallFlags flags,
+			FuFirmwareParseFlags flags,
 			GError **error)
 {
 	g_autoptr(GInputStream) stream = NULL;
@@ -1347,7 +1360,7 @@ fu_firmware_build(FuFirmware *self, XbNode *n, GError **error)
 		if (sz == 0 || sz == G_MAXUINT64) {
 			fu_firmware_set_bytes(self, blob);
 		} else {
-			g_autoptr(GBytes) blob_padded = fu_bytes_pad(blob, (gsize)sz);
+			g_autoptr(GBytes) blob_padded = fu_bytes_pad(blob, (gsize)sz, 0xFF);
 			fu_firmware_set_bytes(self, blob_padded);
 		}
 	}
@@ -1464,17 +1477,22 @@ fu_firmware_build_from_xml(FuFirmware *self, const gchar *xml, GError **error)
 	/* parse XML */
 	if (!xb_builder_source_load_xml(source, xml, XB_BUILDER_SOURCE_FLAG_NONE, error)) {
 		g_prefix_error(error, "could not parse XML: ");
+		fwupd_error_convert(error);
 		return FALSE;
 	}
 	xb_builder_import_source(builder, source);
 	silo = xb_builder_compile(builder, XB_BUILDER_COMPILE_FLAG_NONE, NULL, error);
-	if (silo == NULL)
+	if (silo == NULL) {
+		fwupd_error_convert(error);
 		return FALSE;
+	}
 
 	/* create FuFirmware of specific GType */
 	n = xb_silo_query_first(silo, "firmware", error);
-	if (n == NULL)
+	if (n == NULL) {
+		fwupd_error_convert(error);
 		return FALSE;
+	}
 	return fu_firmware_build(self, n, error);
 }
 
@@ -1508,7 +1526,7 @@ fu_firmware_build_from_filename(FuFirmware *self, const gchar *filename, GError
  * fu_firmware_parse_file:
  * @self: a #FuFirmware
  * @file: a file
- * @flags: install flags, e.g. %FWUPD_INSTALL_FLAG_FORCE
+ * @flags: #FuFirmwareParseFlags, e.g. %FWUPD_INSTALL_FLAG_FORCE
  * @error: (nullable): optional return location for an error
  *
  * Parses a firmware file, typically breaking the firmware into images.
@@ -1518,7 +1536,7 @@ fu_firmware_build_from_filename(FuFirmware *self, const gchar *filename, GError
  * Since: 1.3.3
  **/
 gboolean
-fu_firmware_parse_file(FuFirmware *self, GFile *file, FwupdInstallFlags flags, GError **error)
+fu_firmware_parse_file(FuFirmware *self, GFile *file, FuFirmwareParseFlags flags, GError **error)
 {
 	g_autoptr(GFileInputStream) stream = NULL;
 
@@ -1990,7 +2008,7 @@ fu_firmware_get_images(FuFirmware *self)
 /**
  * fu_firmware_get_image_by_id:
  * @self: a #FuPlugin
- * @id: (nullable): image ID, e.g. `config`
+ * @id: (nullable): image ID, e.g. `config` or `*.mfg|*.elf`
  * @error: (nullable): optional return location for an error
  *
  * Gets the firmware image using the image ID.
@@ -2007,16 +2025,34 @@ fu_firmware_get_image_by_id(FuFirmware *self, const gchar *id, GError **error)
 	g_return_val_if_fail(FU_IS_FIRMWARE(self), NULL);
 	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
 
+	/* non-NULL */
+	if (id != NULL) {
+		g_auto(GStrv) split = g_strsplit(id, "|", 0);
+		for (guint i = 0; i < priv->images->len; i++) {
+			FuFirmware *img = g_ptr_array_index(priv->images, i);
+			for (guint j = 0; split[j] != NULL; j++) {
+				if (g_pattern_match_simple(split[j], fu_firmware_get_id(img)))
+					return g_object_ref(img);
+			}
+		}
+		g_set_error(error,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_FOUND,
+			    "no image id %s found in firmware",
+			    id);
+		return NULL;
+	}
+
+	/* NULL */
 	for (guint i = 0; i < priv->images->len; i++) {
 		FuFirmware *img = g_ptr_array_index(priv->images, i);
-		if (g_strcmp0(fu_firmware_get_id(img), id) == 0)
+		if (fu_firmware_get_id(img) == NULL)
 			return g_object_ref(img);
 	}
-	g_set_error(error,
-		    FWUPD_ERROR,
-		    FWUPD_ERROR_NOT_FOUND,
-		    "no image id %s found in firmware",
-		    id);
+	g_set_error_literal(error,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_FOUND,
+			    "no NULL image id found in firmware");
 	return NULL;
 }
 
@@ -2314,6 +2350,8 @@ fu_firmware_export(FuFirmware *self, FuFirmwareExportFlags flags, XbBuilderNode
 			}
 		}
 		xb_builder_node_insert_text(bn, "data", datastr, "size", dataszstr, NULL);
+	} else if (priv->bytes != NULL && g_bytes_get_size(priv->bytes) == 0) {
+		xb_builder_node_insert_text(bn, "data", NULL, NULL);
 	} else if (priv->bytes != NULL) {
 		gsize bufsz = 0;
 		const guint8 *buf = g_bytes_get_data(priv->bytes, &bufsz);
@@ -2521,7 +2559,7 @@ fu_firmware_new_from_bytes(GBytes *fw)
  * fu_firmware_new_from_gtypes:
  * @stream: a #GInputStream
  * @offset: start offset, useful for ignoring a bootloader
- * @flags: install flags, e.g. %FWUPD_INSTALL_FLAG_IGNORE_CHECKSUM
+ * @flags: install flags, e.g. %FU_FIRMWARE_PARSE_FLAG_IGNORE_CHECKSUM
  * @error: (nullable): optional return location for an error
  * @...: an array of #GTypes, ending with %G_TYPE_INVALID
  *
@@ -2534,7 +2572,7 @@ fu_firmware_new_from_bytes(GBytes *fw)
 FuFirmware *
 fu_firmware_new_from_gtypes(GInputStream *stream,
 			    gsize offset,
-			    FwupdInstallFlags flags,
+			    FuFirmwareParseFlags flags,
 			    GError **error,
 			    ...)
 {
