@@ -609,7 +609,7 @@ static FuFirmware *
 fu_elantp_hid_haptic_device_prepare_firmware(FuDevice *device,
 					     GInputStream *stream,
 					     FuProgress *progress,
-					     FwupdInstallFlags flags,
+					     FuFirmwareParseFlags flags,
 					     GError **error)
 {
 	FuElantpHidHapticDevice *self = FU_ELANTP_HID_HAPTIC_DEVICE(device);
@@ -1094,6 +1094,7 @@ static void
 fu_elantp_hid_haptic_device_set_progress(FuDevice *self, FuProgress *progress)
 {
 	fu_progress_set_id(progress, G_STRLOC);
+	fu_progress_add_step(progress, FWUPD_STATUS_DECOMPRESSING, 0, "prepare-fw");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 5, "detach");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 85, "write");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 2, "attach");
