@@ -871,7 +871,7 @@ static FuFirmware *
 fu_mediatek_scaler_device_prepare_firmware(FuDevice *device,
 					   GInputStream *stream,
 					   FuProgress *progress,
-					   FwupdInstallFlags flags,
+					   FuFirmwareParseFlags flags,
 					   GError **error)
 {
 	g_autoptr(FuFirmware) firmware = fu_mediatek_scaler_firmware_new();
@@ -895,6 +895,7 @@ static void
 fu_mediatek_scaler_device_set_progress(FuDevice *self, FuProgress *progress)
 {
 	fu_progress_set_id(progress, G_STRLOC);
+	fu_progress_add_step(progress, FWUPD_STATUS_DECOMPRESSING, 0, "prepare-fw");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0, "detach");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 99, "write");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 1, "attach");
