@@ -70,7 +70,7 @@ fu_mtd_device_read_firmware(FuDevice *device, FuProgress *progress, GError **err
 	if (!fu_firmware_parse_stream(firmware,
 				      stream_partial,
 				      0x0,
-				      FWUPD_INSTALL_FLAG_NONE,
+				      FU_FIRMWARE_PARSE_FLAG_NONE,
 				      error)) {
 		g_prefix_error(error, "failed to parse image: ");
 		return NULL;
@@ -287,7 +287,11 @@ fu_mtd_device_erase(FuMtdDevice *self, GInputStream *stream, FuProgress *progres
 #ifdef HAVE_MTD_USER_H
 	g_autoptr(FuChunkArray) chunks = NULL;
 
-	chunks = fu_chunk_array_new_from_stream(stream, 0x0, self->erasesize, error);
+	chunks = fu_chunk_array_new_from_stream(stream,
+						FU_CHUNK_ADDR_OFFSET_NONE,
+						FU_CHUNK_PAGESZ_NONE,
+						self->erasesize,
+						error);
 	if (chunks == NULL)
 		return FALSE;
 
@@ -299,6 +303,7 @@ fu_mtd_device_erase(FuMtdDevice *self, GInputStream *stream, FuProgress *progres
 	for (guint i = 0; i < fu_chunk_array_length(chunks); i++) {
 		struct erase_info_user erase = {0x0};
 		g_autoptr(FuChunk) chk = NULL;
+		g_autoptr(FuIoctl) ioctl = fu_udev_device_ioctl_new(FU_UDEV_DEVICE(self));
 
 		/* prepare chunk */
 		chk = fu_chunk_array_index(chunks, i, error);
@@ -306,14 +311,14 @@ fu_mtd_device_erase(FuMtdDevice *self, GInputStream *stream, FuProgress *progres
 			return FALSE;
 		erase.start = fu_chunk_get_address(chk);
 		erase.length = fu_chunk_get_data_sz(chk);
-		if (!fu_udev_device_ioctl(FU_UDEV_DEVICE(self),
-					  2,
-					  (guint8 *)&erase,
-					  sizeof(erase),
-					  NULL,
-					  FU_MTD_DEVICE_IOCTL_TIMEOUT,
-					  FU_UDEV_DEVICE_IOCTL_FLAG_NONE,
-					  error)) {
+		if (!fu_ioctl_execute(ioctl,
+				      2,
+				      (guint8 *)&erase,
+				      sizeof(erase),
+				      NULL,
+				      FU_MTD_DEVICE_IOCTL_TIMEOUT,
+				      FU_IOCTL_FLAG_NONE,
+				      error)) {
 			g_prefix_error(error, "failed to erase @0x%x: ", (guint)erase.start);
 			return FALSE;
 		}
@@ -421,7 +426,11 @@ fu_mtd_device_write_verify(FuMtdDevice *self,
 {
 	g_autoptr(FuChunkArray) chunks = NULL;
 
-	chunks = fu_chunk_array_new_from_stream(stream, 0x0, 10 * 1024, error);
+	chunks = fu_chunk_array_new_from_stream(stream,
+						FU_CHUNK_ADDR_OFFSET_NONE,
+						FU_CHUNK_PAGESZ_NONE,
+						10 * 1024,
+						error);
 	if (chunks == NULL)
 		return FALSE;
 
