@@ -7,6 +7,7 @@
 #include "config.h"
 
 #include <scsi/sg.h>
+#include <stddef.h>
 
 #include "fu-scsi-device.h"
 #include "fu-scsi-struct.h"
@@ -179,7 +180,7 @@ static FuFirmware *
 fu_scsi_device_prepare_firmware(FuDevice *device,
 				GInputStream *stream,
 				FuProgress *progress,
-				FwupdInstallFlags flags,
+				FuFirmwareParseFlags flags,
 				GError **error)
 {
 	g_autoptr(FuFirmware) firmware = fu_firmware_new();
@@ -436,6 +437,7 @@ static void
 fu_scsi_device_set_progress(FuDevice *self, FuProgress *progress)
 {
 	fu_progress_set_id(progress, G_STRLOC);
+	fu_progress_add_step(progress, FWUPD_STATUS_DECOMPRESSING, 0, "prepare-fw");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0, "detach");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 99, "write");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0, "attach");
