@@ -22,7 +22,7 @@
 #define FU_SYNAPTICS_CAPE_DEVICE_HDR_WRITE_DELAY_MS	150  /* ms */
 
 /* defines CAPE command constant values and macro */
-#define FU_SYNAPTICS_CAPE_CMD_WRITE_DATAL_LEN 8	 /* number of guint32 */
+#define FU_SYNAPTICS_CAPE_CMD_WRITE_DATAL_LEN 8 /* number of guint32 */
 
 #define FU_SYNAPTICS_CAPE_CMD_GET_FLAG 0x100 /* GET flag */
 
@@ -284,8 +284,6 @@ fu_synaptics_cape_device_sendcmd_ex(FuSynapticsCapeDevice *self,
 	if (fu_device_has_private_flag(FU_DEVICE(self),
 				       FU_SYNAPTICS_CAPE_DEVICE_FLAG_USE_IN_REPORT_INTERRUPT)) {
 		if (!fu_synaptics_cape_device_get_report_intr(self, st_report, &error_local)) {
-			g_autoptr(FuSynapticsCapeMsg) st_msg2 =
-			    fu_synaptics_cape_cmd_hid_report_get_msg(st_report);
 			/* ignoring io error for software reset command */
 			if ((fu_synaptics_cape_msg_get_cmd_id(st_msg_req) ==
 			     FU_SYNAPTICS_CAPE_CMD_MCU_SOFT_RESET) &&
@@ -505,7 +503,7 @@ static FuFirmware *
 fu_synaptics_cape_device_prepare_firmware(FuDevice *device,
 					  GInputStream *stream,
 					  FuProgress *progress,
-					  FwupdInstallFlags flags,
+					  FuFirmwareParseFlags flags,
 					  GError **error)
 {
 	FuSynapticsCapeDevice *self = FU_SYNAPTICS_CAPE_DEVICE(device);
@@ -537,7 +535,7 @@ fu_synaptics_cape_device_prepare_firmware(FuDevice *device,
 		return NULL;
 
 	/* verify if correct device */
-	if ((flags & FWUPD_INSTALL_FLAG_IGNORE_VID_PID) == 0) {
+	if ((flags & FU_FIRMWARE_PARSE_FLAG_IGNORE_VID_PID) == 0) {
 		const guint16 vid =
 		    fu_synaptics_cape_firmware_get_vid(FU_SYNAPTICS_CAPE_FIRMWARE(firmware));
 		const guint16 pid =
@@ -738,6 +736,7 @@ static void
 fu_synaptics_cape_device_set_progress(FuDevice *self, FuProgress *progress)
 {
 	fu_progress_set_id(progress, G_STRLOC);
+	fu_progress_add_step(progress, FWUPD_STATUS_DECOMPRESSING, 0, "prepare-fw");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 2, "detach");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 94, "write");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 2, "attach");
