@@ -445,7 +445,7 @@ static FuFirmware *
 fu_vli_pd_device_prepare_firmware(FuDevice *device,
 				  GInputStream *stream,
 				  FuProgress *progress,
-				  FwupdInstallFlags flags,
+				  FuFirmwareParseFlags flags,
 				  GError **error)
 {
 	FuVliPdDevice *self = FU_VLI_PD_DEVICE(device);
@@ -880,6 +880,7 @@ static void
 fu_vli_pd_device_set_progress(FuDevice *self, FuProgress *progress)
 {
 	fu_progress_set_id(progress, G_STRLOC);
+	fu_progress_add_step(progress, FWUPD_STATUS_DECOMPRESSING, 0, "prepare-fw");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0, "detach");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_WRITE, 72, "write");
 	fu_progress_add_step(progress, FWUPD_STATUS_DEVICE_RESTART, 0, "attach");
